program RunHiggs
  use Higgs
  use HiggsFixed
  use HiggsEW
  implicit none

  integer :: fd_in = 10, op, nmem

  open(fd_in, file='RunHiggs.in', status='old')

  read(fd_in, '(A50)'), pdfset
  read(fd_in, *), alphas_mZ
  read(fd_in, *), alphas_order
  read(fd_in, *), Ecms
  read(fd_in, *), hadron2
  read(fd_in, *), mH
  read(fd_in, *), mut0
  read(fd_in, *), muh0
  read(fd_in, *), mus0
  read(fd_in, *), muf0
  read(fd_in, *), higgs_order
  read(fd_in, *), sub_order
  read(fd_in, *), mt
  read(fd_in, *), mbmb
  read(fd_in, *), GFew
  close(fd_in)

  call InitQCD
  call InitEW

  call InitPDFSetByName(pdfset)
  call numberPDF(nmem)
  call InitPDF(0)

  call Higgs_UpdateCoeff

  call Run_Single

contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine Run_Single
    implicit none
    real(8) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    real(8) :: rp_fixed(-1:1), rp_onlyth(-1:1), rp_onlyipi(-1:1), rp_ipith(-1:1)
    character*100 :: format_string = '(A30,2X,F8.3,SP,2X,F8.3,2X,F8.3)'

    print '(A8,1X,F6.0,1X,A3)', "sqrtS =", Ecms, "GeV"
    print '(A8,1X,F6.1,1X,A3)', "m_H =", mH, "GeV"
    print *, ""

    call Higgs_scale_error(r_fixed, r_onlyth, r_onlyipi, r_ipith)
    print *, "Cross sections with scale uncertainties (pb)"
    print format_string, "Fixed order:", r_fixed(0), r_fixed(1), r_fixed(-1)
    print format_string, "Fixed order + EW:", &
         r_fixed(0)*(1d0+delta_EW), r_fixed(1)*(1d0+delta_EW), r_fixed(-1)*(1d0+delta_EW)
    print format_string, "Only threshold resummed:", r_onlyth(0), r_onlyth(1), r_onlyth(-1)
    print format_string, "Only pi^2 resummed:", r_onlyipi(0), r_onlyipi(1), r_onlyipi(-1)
    print format_string, "Threshold+pi^2 resummed:", r_ipith(0), r_ipith(1), r_ipith(-1)
    print format_string, "Threshold+pi^2 resummed + EW:", &
         r_ipith(0)*(1d0+delta_EW), r_ipith(1)*(1d0+delta_EW), r_ipith(-1)*(1d0+delta_EW)
    print *, ""

    call Higgs_pdf_error(rp_fixed, rp_onlyth, rp_onlyipi, rp_ipith)
    print *, "Cross sections with PDF uncertainties (pb)"
    print format_string, "Fixed order: ", rp_fixed(0), rp_fixed(1), rp_fixed(-1)
    print format_string, "Fixed order + EW:", &
         rp_fixed(0)*(1d0+delta_EW), rp_fixed(1)*(1d0+delta_EW), rp_fixed(-1)*(1d0+delta_EW)
    print format_string, "Only threshold resummed: ", rp_onlyth(0), rp_onlyth(1), rp_onlyth(-1)
    print format_string, "Only pi^2 resummed: ", rp_onlyipi(0), rp_onlyipi(1), rp_onlyipi(-1)
    print format_string, "Threshold+pi^2 resummed: ", rp_ipith(0), rp_ipith(1), rp_ipith(-1)
    print format_string, "Threshold+pi^2 resummed + EW:", &
         rp_ipith(0)*(1d0+delta_EW), rp_ipith(1)*(1d0+delta_EW), rp_ipith(-1)*(1d0+delta_EW)
    print *, ""
  end subroutine Run_Single

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Helper subroutines

  subroutine Higgs_scale_error(r_fixed, r_onlyth, r_onlyipi, r_ipith)
    implicit none
    real(8), intent(out) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    real(8) :: mut00, muh00, mus00, muf00
    real(8) :: r_leading_central, r_leading_half, r_leading_double
    real(8) :: r_nonsingular_central, r_nonsingular_half, r_nonsingular_double
    real(8) :: e_fixed(2), e_onlyth(2), e_onlyipi(2), e_ipith(2)

    mut00 = mut0
    muh00 = muh0
    mus00 = mus0
    muf00 = muf0

    mut0 = -1
    muh0 = -1
    mus0 = -1
    muf0 = -1
    r_leading_central = Higgs_Leading_CS()
    r_nonsingular_central = Higgs_Nonsingular_CS()
    r_fixed(0) = r_leading_central + r_nonsingular_central
    r_onlyth(0) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central
    r_onlyipi(0) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_central
    r_ipith(0) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central

    muf0 = -0.5d0
    r_leading_half = Higgs_Leading_CS()
    r_nonsingular_half = Higgs_Nonsingular_CS()
    e_fixed(1) = r_leading_half + r_nonsingular_half - r_fixed(0)
    e_onlyth(1) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_half - r_onlyth(0)
    e_onlyipi(1) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_half - r_onlyipi(0)
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_half - r_ipith(0)
    muf0 = -2d0
    r_leading_double = Higgs_Leading_CS()
    r_nonsingular_double = Higgs_Nonsingular_CS()
    e_fixed(2) = r_leading_double + r_nonsingular_double - r_fixed(0)
    e_onlyth(2) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_double - r_onlyth(0)
    e_onlyipi(2) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_double - r_onlyipi(0)
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_double - r_ipith(0)
    muf0 = -1d0
    r_fixed(1) = max(e_fixed(1), e_fixed(2), 0d0)
    r_fixed(-1) = min(e_fixed(1), e_fixed(2), 0d0)
    r_onlyth(1) = max(e_onlyth(1), e_onlyth(2), 0d0)**2
    r_onlyth(-1) = min(e_onlyth(1), e_onlyth(2), 0d0)**2
    r_onlyipi(1) = max(e_onlyipi(1), e_onlyipi(2), 0d0)**2
    r_onlyipi(-1) = min(e_onlyipi(1), e_onlyipi(2), 0d0)**2
    r_ipith(1) = max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = min(e_ipith(1), e_ipith(2), 0d0)**2

    mut0 = -0.5d0
    e_onlyth(1) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    e_onlyipi(1) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_central - r_onlyipi(0)
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    mut0 = -2d0
    e_onlyth(2) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    e_onlyipi(2) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_central - r_onlyipi(0)
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    mut0 = -1d0
    r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
    r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
    r_onlyipi(1) = r_onlyipi(1) + max(e_onlyipi(1), e_onlyipi(2), 0d0)**2
    r_onlyipi(-1) = r_onlyipi(-1) + min(e_onlyipi(1), e_onlyipi(2), 0d0)**2
    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2

    muh0 = -0.5d0
    e_onlyth(1) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    e_onlyipi(1) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_central - r_onlyipi(0)
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    muh0 = -2d0
    e_onlyth(2) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    e_onlyipi(2) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_central - r_onlyipi(0)
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    muh0 = -1d0
    r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
    r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
    r_onlyipi(1) = r_onlyipi(1) + max(e_onlyipi(1), e_onlyipi(2), 0d0)**2
    r_onlyipi(-1) = r_onlyipi(-1) + min(e_onlyipi(1), e_onlyipi(2), 0d0)**2
    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2

    mus0 = -0d0
    e_onlyth(1) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    e_onlyipi(1) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_central - r_onlyipi(0)
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    mus0 = -2d0
    e_onlyth(2) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    e_onlyipi(2) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_central - r_onlyipi(0)
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    mus0 = -1d0
    r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
    r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
    r_onlyipi(1) = r_onlyipi(1) + max(e_onlyipi(1), e_onlyipi(2), 0d0)**2
    r_onlyipi(-1) = r_onlyipi(-1) + min(e_onlyipi(1), e_onlyipi(2), 0d0)**2
    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
    
    r_onlyth(1) = sqrt(r_onlyth(1))
    r_onlyth(-1) = -sqrt(r_onlyth(-1))
    r_onlyipi(1) = sqrt(r_onlyipi(1))
    r_onlyipi(-1) = -sqrt(r_onlyipi(-1))
    r_ipith(1) = sqrt(r_ipith(1))
    r_ipith(-1) = -sqrt(r_ipith(-1))

    mut0 = mut00
    muh0 = muh00
    mus0 = mus00
    muf0 = muf00
  end subroutine Higgs_scale_error


  subroutine Higgs_pdf_error(r_fixed, r_onlyth, r_onlyipi, r_ipith)
    implicit none
    real(8), intent(out) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    real(8) :: r_leading, r_nonsingular
    real(8) :: e_fixed(2), e_onlyth(2), e_onlyipi(2), e_ipith(2)
    integer :: i, j

    r_fixed = 0d0
    r_onlyth = 0d0
    r_onlyipi = 0d0
    r_ipith = 0d0

    r_leading = Higgs_Leading_CS()
    r_nonsingular = Higgs_Nonsingular_CS()
    r_fixed(0) = r_leading + r_nonsingular
    r_onlyth(0) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular
    r_onlyipi(0) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular
    r_ipith(0) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular

    do i = 0, nmem/2-1
       do j = 1, 2
          call InitPDF(i*2+j)
          r_leading = Higgs_Leading_CS()
          r_nonsingular = Higgs_Nonsingular_CS()
          e_fixed(j) = r_leading + r_nonsingular - r_fixed(0)
          e_onlyth(j) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular - r_onlyth(0)
          e_onlyipi(j) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular - r_onlyipi(0)
          e_ipith(j) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular - r_ipith(0)
       enddo
       r_fixed(1) = r_fixed(1) + max(e_fixed(1), e_fixed(2), 0d0)**2
       r_fixed(-1) = r_fixed(-1) + min(e_fixed(1), e_fixed(2), 0d0)**2
       r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
       r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
       r_onlyipi(1) = r_onlyipi(1) + max(e_onlyipi(1), e_onlyipi(2), 0d0)**2
       r_onlyipi(-1) = r_onlyipi(-1) + min(e_onlyipi(1), e_onlyipi(2), 0d0)**2
       r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
       r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
    end do

    r_fixed(1) = sqrt(r_fixed(1))
    r_fixed(-1) = -sqrt(r_fixed(-1))
    r_onlyth(1) = sqrt(r_onlyth(1))
    r_onlyth(-1) = -sqrt(r_onlyth(-1))
    r_onlyipi(1) = sqrt(r_onlyipi(1))
    r_onlyipi(-1) = -sqrt(r_onlyipi(-1))
    r_ipith(1) = sqrt(r_ipith(1))
    r_ipith(-1) = -sqrt(r_ipith(-1))

    call InitPDF(0)
  end subroutine Higgs_pdf_error

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine Run_mH
    implicit none
    real(8) :: r_ipith(-1:1), rp_ipith(-1:1)
    integer :: i
    iERR = 5d-3
    do i = 0, 17
       mH = 115d0 + i*5d0
       call Higgs_scale_error_best(r_ipith)
       call Higgs_pdf_error_best(rp_ipith)
       print *, mH, r_ipith(0)*(1d0+delta_EW), r_ipith(1)*(1d0+delta_EW), r_ipith(-1)*(1d0+delta_EW), &
            rp_ipith(1)*(1d0+delta_EW), rp_ipith(-1)*(1d0+delta_EW)
    end do
  end subroutine Run_mH

  subroutine Run_mH_Band
    implicit none
    real(8) :: r_ipith(-1:1), rp_ipith(-1:1)
    integer :: i
    iERR = 5d-3
    do i = 0, 100
       mH = 100d0 + i
       call Higgs_scale_error_best(r_ipith)
       call Higgs_pdf_error_best(rp_ipith)
       print *, mH, r_ipith(0)*(1d0+delta_EW), r_ipith(1)*(1d0+delta_EW), r_ipith(-1)*(1d0+delta_EW), &
            rp_ipith(1)*(1d0+delta_EW), rp_ipith(-1)*(1d0+delta_EW)
    end do
  end subroutine Run_mH_Band

  subroutine Run_Ecms
    implicit none
    real(8) :: r_ipith(-1:1), rp_ipith(-1:1)
    integer :: i
    iERR = 5d-3
    do i = 0, 35
       Ecms = 7000d0 + i*200d0
       call Higgs_scale_error_best(r_ipith)
       call Higgs_pdf_error_best(rp_ipith)
       print *, Ecms, r_ipith(0)*(1d0+delta_EW), r_ipith(1)*(1d0+delta_EW), r_ipith(-1)*(1d0+delta_EW), &
            rp_ipith(1)*(1d0+delta_EW), rp_ipith(-1)*(1d0+delta_EW)
    end do
  end subroutine Run_Ecms


  subroutine Higgs_scale_error_best(r_ipith)
    implicit none
    real(8), intent(out) :: r_ipith(-1:1)
    real(8) :: mut00, muh00, mus00, muf00
    real(8) :: r_leading_central, r_leading_half, r_leading_double
    real(8) :: r_nonsingular_central, r_nonsingular_half, r_nonsingular_double
    real(8) :: e_ipith(2)

    mut00 = mut0
    muh00 = muh0
    mus00 = mus0
    muf00 = muf0

    mut0 = -1
    muh0 = -1
    mus0 = -1
    muf0 = -1
    r_leading_central = Higgs_Leading_CS()
    r_nonsingular_central = Higgs_Nonsingular_CS()
    r_ipith(0) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central

    muf0 = -0.5d0
    r_leading_half = Higgs_Leading_CS()
    r_nonsingular_half = Higgs_Nonsingular_CS()
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_half - r_ipith(0)
    muf0 = -2d0
    r_leading_double = Higgs_Leading_CS()
    r_nonsingular_double = Higgs_Nonsingular_CS()
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_double - r_ipith(0)
    muf0 = -1d0
    r_ipith(1) = max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = min(e_ipith(1), e_ipith(2), 0d0)**2

    mut0 = -0.5d0
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    mut0 = -2d0
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    mut0 = -1d0
    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2

    muh0 = -0.5d0
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    muh0 = -2d0
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    muh0 = -1d0
    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2

    mus0 = -0d0
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    mus0 = -2d0
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    mus0 = -1d0
    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
    
    r_ipith(1) = sqrt(r_ipith(1))
    r_ipith(-1) = -sqrt(r_ipith(-1))

    mut0 = mut00
    muh0 = muh00
    mus0 = mus00
    muf0 = muf00
  end subroutine Higgs_scale_error_best


  subroutine Higgs_pdf_error_best(r_ipith)
    implicit none
    real(8), intent(out) :: r_ipith(-1:1)
    real(8) :: r_leading, r_nonsingular
    real(8) :: e_ipith(2)
    integer :: i, j

    r_ipith = 0d0

    r_leading = Higgs_Leading_CS()
    r_nonsingular = Higgs_Nonsingular_CS()
    r_ipith(0) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular

    do i = 0, nmem/2-1
       do j = 1, 2
          call InitPDF(i*2+j)
          r_leading = Higgs_Leading_CS()
          r_nonsingular = Higgs_Nonsingular_CS()
          e_ipith(j) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular - r_ipith(0)
       enddo
       r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
       r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
    end do

    r_ipith(1) = sqrt(r_ipith(1))
    r_ipith(-1) = -sqrt(r_ipith(-1))

    call InitPDF(0)
  end subroutine Higgs_pdf_error_best


end program RunHiggs
