#! /usr/bin/env python

import sys
import os
import math

config = {}

################################################################################

pdfsets = ["MSTW2008lo90cl.LHgrid", "MSTW2008nlo90cl.LHgrid", "MSTW2008nnlo90cl.LHgrid"]
alphasMZ = [0.13939, 0.12018, 0.11707]

#pdfsets = ["cteq6ll.LHpdf", "cteq66.LHgrid", "cteq66.LHgrid"]
#alphasMZ = [0.129783, 0.118, 0.118]

config["mt"] = 173.1
config["mbmb"] = 4.2
config["GFew"] = "1.16208d-5"

config["muf"] = -1

################################################################################

config["mut"] = -1
config["muh"] = -1
config["mus"] = -1

config["suborder"] = 3

################################################################################

order_string = ["LO", "NLO", "NNLO"]
config["exefile"] = "./RunHiggs"

def PrepareConfigFile():
    f = open("RunHiggs.in", "w")

    if config["energy"] == 1960:
        config["collider"] = "Tevatron"
        config["hadron2"] = -1
    else:
        config["collider"] = "LHC"
        config["hadron2"] = 1

    string = "%(pdfset)s\n" + "%(alphasMZ)fd0\n" + "%(alorder)d\n" \
        + "%(energy)fd0\n" + "%(hadron2)d\n" \
        + "%(mH)fd0\n" + "%(mut)fd0\n" + "%(muh)fd0\n" + "%(mus)fd0\n" + "%(muf)fd0\n" \
        + "%(order)d\n" + "%(suborder)d\n" \
        + "%(mt)fd0\n" + "%(mbmb)fd0\n" + "%(GFew)s\n"

    f.write(string % config)
    f.close()

def usage(cmd):
    print "Usage: %s sqrtS m_H order pdf_order" % cmd

if __name__ == '__main__':
    try:
        config["energy"] = float(sys.argv[1])
        config["mH"] = float(sys.argv[2])
        config["order"] = int(sys.argv[3])
        pdforder = int(sys.argv[4])
    except (IndexError, ValueError):
        usage(sys.argv[0])
    else:
        if config["order"] < 0 or config["order"] > 2 or pdforder < 0 or pdforder > 2:
            print "Wrong order or pdf_order! Should be 0 or 1 or 2."
        elif config["mH"] < 100. or config["mH"] > 1000.:
            print "Higgs mass excluded!"
        elif config["energy"] < 1960.:
            print "Collider energy too low!"
        else:
            config["pdfset"] = pdfsets[pdforder]
            config["alphasMZ"] = alphasMZ[pdforder]
            config["alorder"] = pdforder
            config["order_string"] = order_string[config["order"]]
            PrepareConfigFile()
            print "************************************************************"
            print "  Higgs production at the %(collider)s in %(order_string)s RG-improved QCD" \
                % config
            print "************************************************************\n"
            os.system("%(exefile)s" % config)

